/****************************************************************************

    This file is part of The Newtonator soft synth.

    Copyright (C) 2010-2011 Michael Bechard <mbechard@users.sourceforge.net>

    The Newtonator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The Newtonator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Newtonator.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/

#include <stk/ADSR.h>
#include "config.h"
#include "newt_lv2_instr.h"
#include "newt_lv2_voice.h"
#include "newt_lv2_instr.peg"

using namespace LV2;

const int NewtonatorInstr::DEFAULT_NUM_VOICES = 3;

int reg()
{
    TRACE("Registering class...");
    TRACE1("p_uri", p_uri);
    return NewtonatorInstr::register_class(p_uri);
}

static int _ = reg();

NewtonatorInstr::NewtonatorInstr(double sr)
  : SynthClass(p_n_ports, p_midi_in)
{
    TRACE("In NewtonatorInstr(sr)...");
    int numVoices = getNumVoices();

    for (int i = 0; i < numVoices; i++)
        m_voices.push_back(new NewtonatorVoice());
    //add_audio_outputs(p_left_out, p_right_out, p_veloc_ext_output_lft, p_veloc_ext_output_rgt);
    add_audio_outputs(p_left_out, p_right_out);

    TRACE1("p_left_out", p_left_out);
    TRACE1("p_right_out", p_right_out);
    TRACE1("m_audio_ports.size()", m_audio_ports.size());
    TRACE1("m_ports.size()", m_ports.size());
    TRACE1("m_audio_ports[0]", m_audio_ports[0]);
    TRACE1("m_audio_ports[1]", m_audio_ports[1]);
}

int NewtonatorInstr::getNumVoices()
{
    //For now, just do the default
    return DEFAULT_NUM_VOICES;
}

unsigned NewtonatorInstr::find_free_voice(unsigned char key, unsigned char velocity)
{
    for (unsigned i = 0; i < m_voices.size(); ++i)
    {
        if (!m_voices[i]->getNewt().isPlaying())
            return i;
    }
    TRACE("find_free_voice failed");
    return 0;
}

void NewtonatorInstr::handle_midi(uint32_t size, unsigned char* data)
{
    if (size != 3) return;
    if (data[0] == 0xe0) //pitch bend
    {
        doPitchBend(data[2]);
    }
    else if (data[0] == 0xb0) //MIDI CTRL
    {
        int val = data[1];
        TRACE1("MIDI CTRL, data[1]", val);
    }
    else
    {
        SynthClass::handle_midi(size, data);
    }
}

void NewtonatorInstr::doPitchBend(stk::StkFloat midiVal)
{
    stk::StkFloat val = (midiVal > 64.0 ? midiVal / 63.5 : .5 + (midiVal / 128.0));
    for (unsigned i = 0; i < m_voices.size(); ++i)
    {
        Newtonator2& newt = m_voices[i]->getNewt();

        if (newt.isPlaying())
        {
            newt.bendFrequency(val);
        }
    }
}
/*void NewtonatorInstr::activate()
{
    TRACE("Before NewtonatorInstr::activate()...");
    SynthClass::activate();
    TRACE("After NewtonatorInstr::activate()...");
}

void NewtonatorInstr::run(uint32_t sample_count)
{
    //TRACE("In NewtonatorInstr::run()...");
}

void NewtonatorInstr::connect_port(uint32_t port, void* data_location)
{
    TRACE1("In NewtonatorInstr::connect_port(), port", port);
    SynthClass::connect_port(port, data_location);
}*/

/*void NewtonatorInstr::pre_process(uint32_t from, uint32_t to)
{
    //Assign all the input ports to their respective member
    //variables on all objects.
    stk::ADSR* envs = NULL;
    VariSource* sources = NULL;

    //TRACE("In NewtonatorInstr::pre_process...");
    for (unsigned i = 0; i < m_voices.size(); ++i)
    {
        Newtonator2& newt = m_voices[i]->getNewt();

        if (newt.isPlaying())
        {
            newt.setAmpRelTime(*p(p_amp_release));
            envs = newt.getAmpEnvs();
            if (envs)
            {
                envs[0].setAllTimes(*p(p_amp_attack), *p(p_amp_decay), *p(p_amp_sustain), *p(p_amp_release));
                envs[1].setAllTimes(*p(p_amp_attack), *p(p_amp_decay), *p(p_amp_sustain), *p(p_amp_release));
            }
            sources = newt.getGravModSources();
            if (sources)
            {
                sources[0].getEnv().setAllTimes(*p(p_gmod_attack), *p(p_gmod_decay), *p(p_gmod_sustain), *p(p_gmod_release));
                sources[1].getEnv().setAllTimes(*p(p_gmod_attack), *p(p_gmod_decay), *p(p_gmod_sustain), *p(p_gmod_release));
                sources[0].setFrequency(*p(p_gmod_freq));
                sources[1].setFrequency(*p(p_gmod_freq));
                switch ((int)*p(p_gmod_wav))
                {
                    case 0:
                        sources[0].setType(VariSource::IMPULSE);
                        sources[1].setType(VariSource::IMPULSE);
                        break;
                    case 1:
                        sources[0].setType(VariSource::NOISE);
                        sources[1].setType(VariSource::NOISE);
                        break;
                    case 2:
                        sources[0].setType(VariSource::SAW);
                        sources[1].setType(VariSource::SAW);
                        break;
                    case 3:
                        sources[0].setType(VariSource::SINE);
                        sources[1].setType(VariSource::SINE);
                        break;
                    case 4:
                        sources[0].setType(VariSource::SQUARE);
                        sources[1].setType(VariSource::SQUARE);
                        break;
                }
            }
            newt.setGravityScale(*p(p_grav_scale));
            newt.setFloorScale(*p(p_floor_scale));
            newt.setClip((*p(p_int_clip) == 1.0f ? true : false));
            newt.setGravModDepth(*p(p_gmod_depth));
            sources = newt.getFloorSources();
            if (sources)
            {
                sources[0].setFrequency(*p(p_floor_freq));
                sources[1].setFrequency(*p(p_floor_freq));
                switch ((int)*p(p_floor_wav))
                {
                    case 0:
                        sources[0].setType(VariSource::IMPULSE);
                        sources[1].setType(VariSource::IMPULSE);
                        break;
                    case 1:
                        sources[0].setType(VariSource::NOISE);
                        sources[1].setType(VariSource::NOISE);
                        break;
                    case 2:
                        sources[0].setType(VariSource::SAW);
                        sources[1].setType(VariSource::SAW);
                        break;
                    case 3:
                        sources[0].setType(VariSource::SINE);
                        sources[1].setType(VariSource::SINE);
                        break;
                    case 4:
                        sources[0].setType(VariSource::SQUARE);
                        sources[1].setType(VariSource::SQUARE);
                        break;
                }
            }
            newt.setVelocReset(*p(p_veloc_reset));
            newt.setGravAdjust((*p(p_grav_adjust) == 1.0f ? true : false));
        }
    }
}
*/
