/****************************************************************************

    This file is part of The Newtonator soft synth.

    Copyright (C) 2010-2011 Michael Bechard <mbechard@users.sourceforge.net>

    The Newtonator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The Newtonator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Newtonator.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/

#include "config.h"
#include "newt_lv2_voice.h"
#include "newt_lv2_instr.peg"
#include <stk/Skini.h>

using namespace LV2;

NewtonatorVoice::NewtonatorVoice()
	: _key(INVALID_KEY)
{
    TRACE("In NewtonatorVoice()...");
    _newt.setSettings(this);
}

void NewtonatorVoice::on(unsigned char key, unsigned char velocity)
{
    if (key == INVALID_KEY)
    {
        //Actually a note-off event
        off(velocity);
    }
    else
    {
        _key = key;
        if (key < 129)
            _newt.setFrequency(stk::Midi2Pitch[key]);
        if (velocity < 128)
            _newt.keyOn(velocity / 127.0f);
    }
}

void NewtonatorVoice::off(unsigned char velocity)
{
    //_key = INVALID_KEY;
    _newt.keyOff(velocity / 127.0f);
}

unsigned char NewtonatorVoice::get_key() const
{
    unsigned char ret = INVALID_KEY;
    //int state = 0;
    //TRACE("In NewtonatorVoice::get_key()...");
    if (_newt.needsKeyOff())
        ret = _key;

    TRACE1("get_key, ret", (int)ret);
    TRACE1("get_key, this", this);
    //TRACE1("get_key, state", state);
    return ret;
}

void NewtonatorVoice::render(uint32_t from, uint32_t to)
{
    float* lft = NULL;
    float* rgt = NULL;
    float tickVal = 0.0;

    if (!_newt.isPlaying())
        return;

    if (_newt.getStereo())
    {
        for (uint32_t i = from; i < to; ++i)
        {
            lft = p(p_left_out) + i;
            rgt = p(p_right_out) + i;

            *lft += _newt.newtTick(0, i);
            *rgt += _newt.newtTick(1, i);
        }
    }
    else
    {
        for (uint32_t i = from; i < to; ++i)
        {
            lft = p(p_left_out) + i;
            rgt = p(p_right_out) + i;

            tickVal = _newt.newtTick(0, i);
            *lft += tickVal;
            *rgt += tickVal;
        }
    }
}
